<?php

namespace PrivateArea;

use PrivateArea\Controller;

if ( ! defined( 'ABSPATH' ) ) die( 'Forbidden' );

/**
 * Access controller class.
 *
 * This class is entitled to determine whether the current user (if any) has
 * access to the screen being loaded on frontend.
 *
 * @since 1.0.0
 */
class AccessController {

	/**
	 * The template name being used to create private area pages.
	 *
	 * @var string
	 */
	public static $template = 'template-private-area.php';

	/**
	 * The controllers being invoked to validate page access.
	 *
	 * @var array
	 */
	private $controllers = [];

	/**
	 * Get the template name used to create private area pages.
	 *
	 * @since 1.0.0
	 * @return string
	 */
	public function get_template() {
		return self::$template;
	}

	/**
	 * Declare the custom page template that will be used to create private area
	 * pages.
	 *
	 * @since 1.0.0
	 */
	public function declare_template() {
		add_filter( 'theme_page_templates', [ $this, 'page_template' ], 999 );
	}

	/**
	 * Add the custom page template that will be used to create private area
	 * pages to the list of available page templates.
	 *
	 * @since 1.0.0
	 * @param array $templates The list of available page templates.
	 * @return array
	 */
	public function page_template( array $templates ) {
		$templates[ self::$template ] = __( 'Private area', 'wp-private-area' );

		return $templates;
	}

	/**
	 * Add a controller to the check options.
	 *
	 * @since 1.0.0
	 * @param Controller $controller The check controller.
	 */
	public function add_controller( Controller $controller ) {
		$this->controllers[ $controller->get_name() ] = $controller;
	}

	/**
	 * Check if a given page belongs to the private area.
	 *
	 * @since 1.0.0
	 * @param \WP_Post|mixed $page The page object.
	 * @return bool
	 */
	public function is_private( $page ) {
		if ( $page instanceof \WP_Post ) {
			$template = \get_post_meta( $page->ID, '_wp_page_template', true );

			return self::$template === $template;
		}

		return false;
	}

	/**
	 * Set the access settings.
	 *
	 * @since 1.0.0
	 * @param array $settings The settings array.
	 */
	public function set_settings( array $settings ) {
		$this->settings = $settings;
	}

	/**
	 * Return the access details configuration for the specified page.
	 *
	 * @since 1.0.0
	 * @param int $page_id The page ID.
	 * @return array
	 */
	private function page_access_details( int $page_id ) {
		$access_level = \PrivateArea\Core::get_instance()->get_settings()->get( 'access_level', [] );
		$page = get_post( $page_id );

		if ( isset( $access_level[ $page_id ] ) ) {
			if ( $page->post_parent ) {
				if ( ! in_array( $access_level[ $page_id ][ 'criterion' ], array_keys( $this->controllers ) ) ) {
					return $this->page_access_details( absint( $page->post_parent ) );
				}
			}

			return $access_level[ $page_id ];
		}
		else {
			if ( $page->post_parent ) {
				return $this->page_access_details( absint( $page->post_parent ) );
			}
		}

		return [
			'criterion' => current( array_keys( $this->controllers ) )
		];
	}

	/**
	 * Check if the current user is entitled to access the current screen.
	 *
	 * @since 1.0.0
	 */
	public function check() {
		if ( \is_admin() ) {
			return false;
		}

		if ( defined( 'DOING_AJAX' ) && DOING_AJAX ) {
			return false;
		}

		$page = \get_queried_object();

		if ( ! $page ) {
			return false;
		}

		$check = false;

		if ( $this->controllers ) {
			$settings = $this->page_access_details( $page->ID );

			if ( ! isset( $settings[ 'criterion' ] ) ) {
				return false;
			}

			if ( isset( $this->controllers[ $settings[ 'criterion' ] ] ) ) {
				$controller = $this->controllers[ $settings[ 'criterion' ] ];
			}
			else {
				$controller = current( $this->controllers );
			}

			$controller->set_settings( $settings );

			$check = (bool) $controller->check();
		}

		return $check;
	}

}
