<?php

namespace PrivateArea;

if ( ! defined( 'ABSPATH' ) ) die( 'Forbidden' );

/**
 * Access handler class.
 *
 * This class is entitled to handle the rejection to access a private page.
 *
 * @since 1.0.0
 */
class AccessHandler {

	/**
	 * Accept access to a private page.
	 *
	 * @since 1.0.0
	 */
	public function accept() {
	}

	/**
	 * Reject access to a private page.
	 *
	 * @since 1.0.0
	 */
	public function reject() {
		/* Output the rejection header. */
		$this->rejection_header();

		add_filter( 'the_content', function( $content ) {
			$rejection_text = '<p>' . esc_html__( 'You are not allowed to view this page.', 'wp-private-area' ) . '</p>';
			$rejection_text = apply_filters( 'wp_private_area_rejection_text', $rejection_text );

			return $rejection_text;
		}, 9999 );
	}

	/**
	 * Output the rejection header.
	 *
	 * @since 1.0.0
	 */
	private function rejection_header() {
		$protocol = ( isset( $_SERVER[ 'SERVER_PROTOCOL' ] ) ? $_SERVER[ 'SERVER_PROTOCOL' ] : 'HTTP/1.0' );
		$code = 403;
		$text = 'Forbidden';

		header( $protocol . ' ' . $code . ' ' . $text );
	}

	/**
	 * Output the noindex meta tag to private pages.
	 *
	 * @since 1.0.0
	 */
	public function noindex() {
		echo '<meta name="robots" content="noindex,nofollow">';
	}

	/**
	 * Output the noindex meta tag to private pages.
	 *
	 * @since 1.0.0
	 * @param string $output The output of the robots.txt file.
	 * @return string
	 */
	public function robots( $output ) {
		$instance = Core::get_instance();
		$pages = $instance->get_pages();

		foreach ( $pages as $page ) {
			$link = wp_make_link_relative( get_permalink( $page[ 'id' ] ) );

			$output  .= "Disallow: ${link}\n";
		}

		return $output;
	}

	/**
	 * Exclude pages from the sitemap.
	 *
	 * @since 1.0.0
	 */
	public function exclude_from_sitemap() {
		$instance = Core::get_instance();
		$pages = $instance->get_pages();
		$page_ids = \wp_list_pluck( $pages, 'id' );

		/* Exclude from sitemaps generated with YOAST. */
		add_filter( 'wpseo_exclude_from_sitemap_by_post_ids', function( $ids ) use ( $page_ids ) {
			$ids = array_merge( $ids, $page_ids );
			$ids = array_unique( $ids );

			return $ids;
		} );

		/* Exclude from sitemaps generated with Google XML Sitemaps. */
		add_filter( 'option_sm_options', function( $value ) use ( $page_ids ) {
			if ( is_admin() ) {
				return $value;
			}

			$value[ 'sm_b_exclude' ] = array_merge( $value[ 'sm_b_exclude' ], $page_ids );
			$value[ 'sm_b_exclude' ] = array_unique( $value[ 'sm_b_exclude' ] );

			return $value;
		} );

		/* Exclude from sitemaps generated with All in one SEO pack. */
		add_filter( 'option_aioseop_options', function( $value ) use ( $page_ids ) {
			if ( is_admin() ) {
				return $value;
			}

			if ( isset( $value[ 'modules' ][ 'aiosp_sitemap_options' ] ) ) {
				$pages = explode( ',', $value[ 'modules' ][ 'aiosp_sitemap_options' ][ 'aiosp_sitemap_excl_pages' ] );
				$pages = array_merge( $pages, $page_ids );
				$pages = array_unique( $pages );

				$value[ 'modules' ][ 'aiosp_sitemap_options' ][ 'aiosp_sitemap_excl_pages' ] = implode( ',', $pages );
				$value[ 'modules' ][ 'aiosp_sitemap_options' ][ 'aiosp_sitemap_excl_pages' ] = trim( $value[ 'modules' ][ 'aiosp_sitemap_options' ][ 'aiosp_sitemap_excl_pages' ], ',' );
			}

			return $value;
		} );
	}

}
