<?php

namespace PrivateArea\Controllers;

use PrivateArea\Controller as Controller;

if ( ! defined( 'ABSPATH' ) ) die( 'Forbidden' );

/**
 * Role controller class.
 *
 * This class implements a role-based controller, that is a controller that
 * allow only logged-in users to pass the check, or users with a specific role.
 *
 * @since 1.0.0
 */
class RoleController extends Controller {

	/**
	 * The user being tested for access credentials.
	 *
	 * @var WP_User
	 */
	private $user = null;

	/**
	 * The capabilities required for the user to pass the access test.
	 *
	 * @var array
	 */
	private $capabilities = [];

	/**
	 * Class constructor.
	 *
	 * @since 1.0.0
	 */
	public function __construct() {
		\add_filter( 'wp_private_area_access_criteria', function( $criteria ) {
			$criteria[ $this->get_name() ] = __( 'Specific role(s)', 'wp-private-area' );

			return $criteria;
		} );

		\add_action( 'wp_private_area_criterion_value_role', function( $data, $page ) {
			$roles = $this->load_roles();
			$access_roles = isset( $data[ 'settings' ][ 'access_level' ][ 'value' ][ $page[ 'id' ] ][ 'role' ] ) ? (array) $data[ 'settings' ][ 'access_level' ][ 'value' ][ $page[ 'id' ] ][ 'role' ] : [];
			?>
			<div data-private-area-access-controller-value="role">
				<select class="private-area-multiselect" name="<?php echo esc_attr( $this->access_level_name( intval( $page[ 'id' ] ), '[]' ) ) ?>" multiple="multiple">
					<?php foreach ( $roles as $k => $l ) : ?>
						<option value="<?php echo esc_attr( $k ); ?>" <?php echo in_array( $k, $access_roles ) ? 'selected' : ''; ?>><?php echo esc_html( $l ); ?></option>
					<?php endforeach; ?>
				</select>

				<span class="private-area-help" title="<?php esc_attr_e( 'Administrators have access to all pages by default.', 'wp-private-area' ); ?>"><span class="screen-reader-text"><?php esc_html_e( 'Help', 'wp-private-area' ); ?></span></span>
			</div>
			<?php
		}, 10, 2 );

		\add_filter( 'wp_private_area_setting_value_role', function( $value ) {
			$value = array_map( 'sanitize_text_field', (array) $value );

			return $value;
		} );
	}

	/**
	 * Set the user being tested for access credentials.
	 *
	 * @since 1.0.0
	 * @param \WP_User $user
	 */
	public function set_user( \WP_User $user ) {
		$this->user = $user;
	}

	/**
	 * Set the capabilities required for the user to pass the access test.
	 *
	 * @since 1.0.0
	 * @param array $user
	 */
	public function set_capabilities( array $capabilities ) {
		if ( ! $capabilities ) {
			$this->capabilities = array_keys( $this->load_roles() );
		}
		else {
			$this->capabilities = $capabilities;
		}
	}

	/**
	 * Load the currently available roles.
	 *
	 * @since 1.0.0
	 * @return array
	 */
	private function load_roles() {
		global $wp_roles;

		if ( ! isset( $wp_roles ) ) {
			$wp_roles = new \WP_Roles();
		}

		$roles = $wp_roles->get_names();

		if ( isset( $roles[ 'administrator' ] ) ) {
			unset( $roles[ 'administrator' ] );
		}

		return $roles;
	}

	/**
	 * Get the name of the controller.
	 *
	 * @since 1.0.0
	 * @return string
	 */
	public function get_name() {
		return 'role';
	}

	/**
	 * Setup the controller data.
	 *
	 * @since 1.0.0
	 */
	public function setup() {
		$this->set_user( \wp_get_current_user() );

		if ( isset( $this->settings[ 'role' ] ) ) {
			$this->set_capabilities( (array) $this->settings[ 'role' ] );
		}
	}

	/**
	 * Check if the current visitor is entitled to access the current screen.
	 *
	 * @since 1.0.0
	 * @return bool
	 */
	public function check() {
		$this->setup();

		if ( ! $this->user || ! $this->user instanceof \WP_User || ! $this->user->ID ) {
			return false;
		}

		/* Always grant access to administrators. */
		$this->capabilities[] = 'administrator';

		foreach ( $this->capabilities as $cap ) {
			if ( $this->user->has_cap( $cap ) ) {
				return true;
			}
		}

		return false;
	}

}
