<?php

namespace PrivateArea;

use PrivateArea\Settings;
use PrivateArea\Controllers\RoleController;
use PrivateArea\Controllers\DirectAccessController;

if ( ! defined( 'ABSPATH' ) ) die( 'Forbidden' );

/**
 * Main plugin class.
 *
 * @since 1.0.0
 */
class Core {

	/**
	 * Class instance.
	 *
	 * @var PrivateArea\Core
	 */
	private static $instance = null;

	/**
	 * The plugin version.
	 *
	 * @var string
	 */
	private $version = '';

	/**
	 * The plugin folder path.
	 *
	 * @var string
	 */
	private $path = '';

	/**
	 * The plugin URI.
	 *
	 * @var string
	 */
	private $uri = '';

	/**
	 * Access controller.
	 *
	 * @var AccessController
	 */
	private $access_controller = null;

	/**
	 * Access handler.
	 *
	 * @var AccessHandler
	 */
	private $access_handler = null;

	/**
	 * Settings controller.
	 *
	 * @var Settings
	 */
	private $settings = null;

	/**
	 * Return the class instance.
	 *
	 * @since 1.0.0
	 * @return Core
	 */
	public static function get_instance() {
		if ( null === self::$instance ) {
			self::$instance = new self();
		}

		return self::$instance;
	}

	/**
	 * Set the plugin folder path.
	 *
	 * @since 1.0.0
	 * @param string $path The plugin folder path.
	 */
	public function set_path( string $path ) {
		$this->path = $path;
	}

	/**
	 * Set the plugin URI.
	 *
	 * @since 1.0.0
	 * @param string $uri The plugin URI.
	 */
	public function set_uri( string $uri ) {
		$this->uri = $uri;
	}

	/**
	 * Set the plugin version.
	 *
	 * @since 1.0.0
	 * @param string $version The plugin version.
	 */
	public function set_version( string $version ) {
		$this->version = $version;
	}

	/**
	 * Set the access controller.
	 *
	 * @since 1.0.0
	 * @param AccessController $access_controller The access controller instance.
	 */
	public function set_access_controller( AccessController $access_controller ) {
		$this->access_controller = $access_controller;
	}

	/**
	 * Set the access handler.
	 *
	 * @since 1.0.0
	 * @param AccessHandler $access_handler The access handler instance.
	 */
	public function set_access_handler( AccessHandler $access_handler ) {
		$this->access_handler = $access_handler;
	}

	/**
	 * Set the settings controller.
	 *
	 * @since 1.0.0
	 * @param Settings $settings The settings controller instance.
	 */
	public function set_settings( Settings $settings ) {
		$this->settings = $settings;
	}

	/**
	 * Get the plugin folder path.
	 *
	 * @since 1.0.0
	 * @return string
	 */
	public function get_path() {
		return $this->path;
	}

	/**
	 * Get the plugin URI.
	 *
	 * @since 1.0.0
	 * @return string
	 */
	public function get_uri() {
		return $this->uri;
	}

	/**
	 * Get the plugin version.
	 *
	 * @since 1.0.0
	 * @return string
	 */
	public function get_version() {
		return $this->version;
	}

	/**
	 * Get the access controller.
	 *
	 * @since 1.0.0
	 * @return AccessController
	 */
	public function get_access_controller() {
		return $this->access_controller;
	}

	/**
	 * Get the access handler.
	 *
	 * @since 1.0.0
	 * @return AccessHandler
	 */
	public function get_access_handler() {
		return $this->access_handler;
	}

	/**
	 * Get the settings controller.
	 *
	 * @since 1.0.0
	 * @return Settings
	 */
	public function get_settings() {
		return $this->settings;
	}

	/**
	 * Get the list of pages that compose the private area.
	 *
	 * @since 1.0.0
	 * @return array
	 */
	public function get_pages() {
		$ps = \get_posts( [
			'post_status'    => 'any',
			'post_type'      => 'page',
			'meta_key'       => '_wp_page_template',
			'meta_value'     => $this->access_controller::$template,
			'posts_per_page' => -1
		] );

		$pages = [];

		foreach ( $ps as $p ) {
			$pages[] = [
				'title'  => $p->post_title,
				'parent' => $p->post_parent,
				'id'     => $p->ID
			];
		}

		return $pages;
	}

	/**
	 * Lock private area pages.
	 *
	 * @since 1.0.0
	 */
	public function lock_pages() {
		/* Hide pages from the robots.txt file. */
		add_filter( 'robots_txt', [ $this->access_handler, 'robots' ] );

		/* Exclude pages from the sitemap. */
		add_action( 'init', [ $this->access_handler, 'exclude_from_sitemap' ] );

		/* Add the default controller. */
		$this->access_controller->add_controller( new RoleController() );

		add_action( 'template_redirect', function() {
			$page = \get_queried_object();

			if ( ! $this->access_controller->is_private( $page ) ) {
				return;
			}

			/* Add a noindex meta tag for the current page. */
			add_action( 'wp_head', [ $this->access_handler, 'noindex' ] );

			if ( ! $this->access_controller->check() ) {
				$this->access_handler->reject();
			}
			else {
				$this->access_handler->accept();
			}
		}, 999 );
	}

	/**
	 * Conditionally disable the admin bar for logged-in non administrators.
	 *
	 * @since 1.0.0
	 */
	public function maybe_disable_admin_bar() {
		add_filter( 'show_admin_bar', function( $display ) {
			if ( ! current_user_can( 'administrator' ) && $this->get_settings()->disable_admin_bar() ) {
				return false;
			}

			return $display;
		}, 9999 );
	}

	/**
	 * Conditionally redirect users after logging in.
	 *
	 * @since 1.0.0
	 */
	public function maybe_redirect_after_login() {
		add_filter( 'login_redirect', function( $redirect_to, $request, $user ) {
			if( $user && is_object( $user ) && is_a( $user, 'WP_User' ) ) {
				if( ! $user->has_cap( 'administrator' ) ) {
					$redirect = $this->get_settings()->login_redirect_to();

					if ( false !== $redirect ) {
						$redirect_to = $redirect;
					}
				}
			}

			return $redirect_to;
		}, 9999, 3 );
	}

}
