<?php

namespace PrivateArea;

use PrivateArea\Utils\Asset;

if ( ! defined( 'ABSPATH' ) ) die( 'Forbidden' );

/**
 * Settings class.
 *
 * This class is entitled to create the plugin's settings page as well as retrieve
 * settings.
 *
 * @since 1.0.0
 */
class Settings {

	/**
	 * Create the settings page on admin.
	 *
	 * @since 1.0.0
	 */
	public function create_page() {
		\add_action( 'admin_menu', function() {
			\add_submenu_page(
				'options-general.php',
				__( 'Settings', 'wp-private-area' ),
				__( 'Private area', 'wp-private-area' ),
				'manage_options',
				'private-area-settings',
				function() {
					$this->save();

					$pages = \PrivateArea\Core::get_instance()->get_pages();

					$data = [
						'pages'    => $pages,
						'settings' => $this->load_settings( $pages ),
						'criteria' => $this->load_access_criteria()
					];

					require_once Core::get_instance()->get_path() . 'admin/settings.php';
				}
			);
		} );

		\add_action( 'admin_enqueue_scripts', function() {
			$instance = Core::get_instance();

			Asset::register_script(
				'notices',
				'admin/assets/js/notices.js',
				[ 'jquery' ],
				$instance->get_version()
			);

			Asset::enqueue_script( 'notices' );

			if ( ! isset( $_GET[ 'page' ] ) || 'private-area-settings' !== $_GET[ 'page' ] ) {
				return;
			}

			Asset::register_style(
				'select2',
				'admin/assets/css/select2.min.css',
				[],
				$instance->get_version()
			);

			Asset::register_script(
				'select2',
				'admin/assets/js/select2.full.min.js',
				[],
				$instance->get_version()
			);

			Asset::register_style(
				'tiptip',
				'admin/assets/css/tiptip.css',
				[],
				$instance->get_version()
			);

			Asset::register_script(
				'tiptip',
				'admin/assets/js/jquery.tipTip.minified.js',
				[ 'jquery' ],
				$instance->get_version()
			);

			Asset::register_style(
				'admin',
				'admin/assets/css/private-area.css',
				[ 'wp-private-area-select2', 'wp-private-area-tiptip' ],
				$instance->get_version()
			);

			Asset::register_script(
				'admin',
				'admin/assets/js/private-area.js',
				[ 'jquery', 'wp-private-area-select2', 'wp-private-area-tiptip' ],
				$instance->get_version()
			);

			Asset::enqueue_style( 'admin' );
			Asset::enqueue_script( 'admin' );

			// wp_enqueue_style( 'wp-private-area-select2', $instance->get_uri() . 'admin/assets/css/select2.min.css', null, $instance->get_version(), false );
			// wp_enqueue_script( 'wp-private-area-select2', $instance->get_uri() . 'admin/assets/js/select2.full.min.js', null, $instance->get_version(), true );

			// wp_enqueue_style( 'wp-private-area-tiptip', $instance->get_uri() . 'admin/assets/css/tiptip.css', null, $instance->get_version(), false );
			// wp_enqueue_script( 'wp-private-area-tiptip', $instance->get_uri() . 'admin/assets/js/jquery.tipTip.minified.js', [ 'jquery' ], $instance->get_version(), true );

			// wp_enqueue_style( 'wp-private-area-admin', $instance->get_uri() . 'admin/assets/css/private-area.css', [ 'wp-private-area-select2', 'wp-private-area-tiptip' ], $instance->get_version(), false );
			// wp_enqueue_script( 'wp-private-area-admin', $instance->get_uri() . 'admin/assets/js/private-area.js', [ 'jquery', 'wp-private-area-select2', 'wp-private-area-tiptip' ], $instance->get_version(), true );
		} );
	}

	/**
	 * List currently available access criteria.
	 *
	 * @since 1.0.0
	 * @return array
	 */
	private function load_access_criteria() {
		return apply_filters( 'wp_private_area_access_criteria', [] );
	}

	/**
	 * Load the plugin settings for editing.
	 *
	 * @since 1.0.0
	 * @param array $pages The array of private area pages.
	 * @return array
	 */
	private function load_settings( array $pages = [] ) {
		$settings = [
			'login_redirect_to' => [
				'value' => $this->get( 'login_redirect_to', '0' ),
				'options' => []
			],
			'admin_bar_disable' => [
				'value' => $this->get( 'admin_bar_disable', '1' )
			],
			'access_level' => [
				'value' => $this->get( 'access_level', [] )
			]
		];

		foreach ( $pages as $page ) {
			if ( ! $page[ 'parent' ] ) {
				$settings[ 'login_redirect_to' ][ 'options' ][ $page[ 'id' ] ] = $page[ 'title' ];
			}
		}

		return $settings;
	}

	/**
	 * Save the plugin settings.
	 *
	 * @since 1.0.0
	 */
	private function save() {
		if ( ! current_user_can( 'manage_options' ) ) {
			wp_die( 'Unauthorized user' );
		}

		if ( empty( $_POST ) ) {
			return;
		}

		\check_admin_referer( 'wp_private_area_settings' );

		$login_redirect_to = isset( $_POST[ 'login_redirect_to' ] ) ? sanitize_text_field( $_POST[ 'login_redirect_to' ] ) : 0;
		$admin_bar_disable = isset( $_POST[ 'admin_bar_disable' ] ) ? intval( sanitize_text_field( $_POST[ 'admin_bar_disable' ] ) ) : 0;
		$access_level = [];

		if ( isset( $_POST[ 'access_level' ] ) ) {
			$criteria = $this->load_access_criteria();

			foreach ( $_POST[ 'access_level' ] as $page_id => $data ) {
				$page_id   = absint( $page_id );

				$access_level[ $page_id ] = [
					'criterion' => sanitize_text_field( $data[ 'criterion' ] )
				];

				foreach ( $criteria as $k => $v ) {
					$access_level[ $page_id ][ $k ] = apply_filters(
						'wp_private_area_setting_value_' . $k,
						isset( $data[ $k ] ) ? $data[ $k ] : ''
					);
				}
			}
		}

		update_option( 'wp_private_area_login_redirect_to', $login_redirect_to );
		update_option( 'wp_private_area_admin_bar_disable', $admin_bar_disable );
		update_option( 'wp_private_area_access_level', $access_level );
	}

	/**
	 * Retrieve a setting.
	 *
	 * @since 1.0.0
	 * @param string $key The setting key.
	 * @param mixed $default The setting default value.
	 * @return mixed
	 */
	public function get( string $key, $default = null ) {
		return get_option( 'wp_private_area_' . $key, $default );
	}

	/**
	 * Check if we should disable the admin bar display for logged-in non administrators.
	 *
	 * @since 1.0.0
	 * @return bool
	 */
	public function disable_admin_bar() {
		$admin_bar_disable = $this->get( 'admin_bar_disable', false );

		return (bool) $admin_bar_disable;
	}

	/**
	 * The URL we should redirect users to after a login.
	 *
	 * @since 1.0.0
	 * @return string|bool
	 */
	public function login_redirect_to() {
		$redirect = $this->get( 'login_redirect_to', false );

		switch ( $redirect ) {
			case is_numeric( $redirect ):
				$redirect = get_permalink( absint( $redirect ) );
				break;
			case 'home':
				$redirect = home_url();
				break;
			default:
				$redirect = false;
				break;
		}

		return $redirect;
	}

}
